'use strict';
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');
var templateParams = require('../templateParams.json');
var file = require('fs');

module.exports = yeoman.Base.extend({
  prompting: function () {
    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the tremendous ' + chalk.red('generator-thor-module') + ' generator!'
    ));

    var prompts = [
    {
        type: 'list',
        name: 'moduleType',
        message: 'What would you like to write module types with?',
        choices: ['Built-In', 'External']
    },
    {
        type: 'confirm',
        name: 'routing',
        message: 'Would you like to add route interface?',
        default: true
    }];

    return this.prompt(prompts).then(function (props) {
      this.props = props;
    }.bind(this));
  },

  writing: function () {
    var params = templateParams.params;
    var mainParam = Object.assign(params.main, {modname: this.appname});
    var protoParam = Object.assign(params.proto, {modname: this.appname});
    var defNameParam = Object.assign(params.defName, {modname: this.appname});

    var makefilePath = (this.props.moduleType === 'Built-In') ? 'makefiles/builtin/Makefile' : 'makefiles/external/Makefile';
    var mainfilePath = this.props.routing ? 'src/main_routing.c' : 'src/main.c';
    var protofilePath = this.props.routing ? 'include/proto_routing.h' : 'include/proto_routing.h';
    var protoDstPath = 'include/OFKProto'+this.appname+'.h';
    
    var makefileData = [ makefilePath, 'Makefile', { modname: this.appname } ];
    var mainData = [ mainfilePath, 'src/OFK' + this.appname + '.c', mainParam ];
    var defnameData = ['src/defname.h', 'src/OFKDefIfName.h', defNameParam ];
    var headerData = [ 'include/header.h', 'include/OFK' + this.appname + '.h', { modname: this.appname} ];
    var protoData = [ protofilePath, protoDstPath, protoParam ];

    var tplData = [];

    if (!process.env.THOR_ROOT || !process.env.TARGET_DIR) {
      console.log("Error: Environment failed: THOR_ROOT or TARGET_DIR can not found.");
      return;
    }
    
    tplData.push(makefileData);
    tplData.push(mainData);
    tplData.push(defnameData);
    tplData.push(headerData);
    tplData.push(protoData);

    tplData.forEach((e)=>{
      this.fs.copyTpl( this.templatePath(e[0]), this.destinationPath(e[1]), e[2] );
    });

    var watcher = file.watchFile(protoDstPath, { interval: 1000 },(event, filename) => {
        if (event.size > 0) {
          this.composeWith('thor-module:genProto', { options: { answer: true } });
          file.unwatchFile(protoDstPath);
        }
    });
  }
});
