'use strict';
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');
var templateParams = require('../templateParams.json');
var format = require('string-format');
var util = require('../util');

module.exports = yeoman.Base.extend({
  prompting: function () {
    // Have Yeoman greet the user.

    var prompts = [{
      type: 'confirm',
      name: 'answer',
      message: 'Would you like to register a builtin module?',
      default: true
    }];

    return this.prompt(prompts).then(function (props) {
      // To access props later use this.props.someAnswer;
      this.props = props;
    }.bind(this));
  },
  writing: function () {
    var params = templateParams.params;
    var thorHeaderParam = params.thorHeader;
    var moduleConfParam = params.moduleConf;
    var glbMakefileParam = params.glbMakefile;

    if (!process.env.THOR_ROOT || !process.env.TARGET_DIR) {
      console.log("Error: Environment failed: THOR_ROOT or TARGET_DIR can not found.");
      return;
    }

    var thorHeaderPath = process.env.THOR_ROOT + '/common/modules/include/OFKThor.h';
    var moduConfPath = process.env.THOR_ROOT + "/" + process.env.TARGET_DIR + "/sys/OFPModuleConfig.c";
    var glbMakefilePath = process.env.THOR_ROOT + "/" + process.env.TARGET_DIR + "/Makefile";
    var curModPath = process.cwd();
    var curModName = curModPath.substring(curModPath.search('modules'), curModPath.length); 

    var loadModFormatStr = 'mId =OFKIF_knlLoadModule("{0}", 0, NULL, -1, NULL);\n\
    if (mId < 0)\n\t{\n\
    \t__OFK_knlPrintf(knlIf, "[warning] not found {1}...\\n");\n\t}\n\
    OFKIF_knlWaitResource(mId, OFK_RSAVIL_OPEN, -1);\n'
                       
    loadModFormatStr = format(loadModFormatStr, this.appname, this.appname);

    var curIncPath = curModPath.substring(curModPath.search('common'), curModPath.length);
    curIncPath = '#include "../../../' + curIncPath + '/include/OFK' + this.appname + '.h"'

    thorHeaderParam.OFK_INCMOD_DECLARE = curIncPath;

    moduleConfParam.OFK_IMPORT_DECLARE = "OFK_IMPORT_MODULE(" + this.appname + ")";
    moduleConfParam.OFK_BLTIN_LIST_DECLARE = "\tOFK_MODULE(" + this.appname + ")";
    moduleConfParam.OFK_MODCONF_DECLARE= "\t" + loadModFormatStr;

    glbMakefileParam.OFK_MAKELIB_DECLARE = "\toutput/" + this.appname + ".a \\";
    glbMakefileParam.OFK_MAKESUBDIR_DECLARE = "\t../../common/" + curModName + " \\";

    if (this.props.answer == false)
      return;

    util.copyfileCmtTpl(this.fs, thorHeaderPath, thorHeaderPath, thorHeaderParam);
    util.copyfileCmtTpl(this.fs, moduConfPath, moduConfPath, moduleConfParam);
    util.copyfileCmtTpl(this.fs, glbMakefilePath, glbMakefilePath, glbMakefileParam, "#");
  }
});
